# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8-80 compliant>

from bpy.types import (
    Operator,
    OperatorFileListElement,
)
from bpy_extras.io_utils import (
    ImportHelper,
    ExportHelper,
    orientation_helper,
    axis_conversion,
)
from bpy.props import (
    StringProperty,
    BoolProperty,
    CollectionProperty,
    EnumProperty,
    FloatProperty,
)
import bpy
import os
bl_info = {
    "name": "Rheo STL format",
    "author": "Rheologic",
    "version": (1, 1, 4),
    "blender": (3, 0, 0),
    "location": "File > Import-Export > Rheo Stl",
    "description": "Export STL files",
    "warning": "",
    "category": "Export",
}


"""
Import-Export STL files (binary or ascii)

- Import automatically remove the doubles.
- Export can export with/without modifiers applied

Issues:

Import:
    - Does not handle endien
"""

if "bpy" in locals():
    import importlib
    if "rheo_stl_utils" in locals():
        importlib.reload(rheo_stl_utils)
    if "blender_utils" in locals():
        importlib.reload(blender_utils)


@orientation_helper(axis_forward='Y', axis_up='Z')
class Export_RHEO_STL(Operator, ExportHelper):
    """Save STL triangle mesh data from the active object"""
    bl_idname = "rheo_export_mesh.stl"
    bl_label = "Export STL"

    filename_ext = ".stl"
    filter_glob: StringProperty(default="*.stl")

    use_selection: BoolProperty(
        name="Selection Only",
        description="Export selected objects only",
        default=True,
    )
    use_prefix: BoolProperty(
        name="use prefix",
        description="Export selected objects with prefix (filename)",
        default=True,
    )
    global_scale: FloatProperty(
        name="Scale",
        min=0.01, max=1000.0,
        default=1.0,
    )

    use_scene_unit: BoolProperty(
        name="Scene Unit",
        description="Apply current scene's unit (as defined by unit scale) to exported data",
        default=False,
    )
    ascii: BoolProperty(
        name="Ascii",
        description="Save the file in ASCII file format",
        default=True,
    )
    use_mesh_modifiers: BoolProperty(
        name="Apply Modifiers",
        description="Apply the modifiers before saving",
        default=True,
    )
    batch_mode: EnumProperty(
        name="Batch Mode",
        items=[('OFF', "Off", "All data in one file"),
               ('OBJECT', "Object", "Each object as a file")],
        default='OBJECT'
    )

    @property
    def check_extension(self):
        return self.batch_mode == 'OFF'

    def execute(self, context):
        from . import rheo_stl_utils
        from . import blender_utils
        import itertools
        from mathutils import Matrix
        keywords = self.as_keywords(ignore=("axis_forward",
                                            "axis_up",
                                            "use_selection",
                                            "use_prefix",
                                            "global_scale",
                                            "check_existing",
                                            "filter_glob",
                                            "use_scene_unit",
                                            "use_mesh_modifiers",
                                            "batch_mode"
                                            ))

        scene = context.scene
        if self.use_selection:
            data_seq = context.selected_objects
        else:
            data_seq = scene.objects

        # Take into account scene's unit scale, so that 1 inch in Blender gives 1 inch elsewhere! See T42000.
        global_scale = self.global_scale
        if scene.unit_settings.system != 'NONE' and self.use_scene_unit:
            global_scale *= scene.unit_settings.scale_length

        global_matrix = axis_conversion(to_forward=self.axis_forward,
                                        to_up=self.axis_up,
                                        ).to_4x4() @ Matrix.Scale(global_scale, 4)

        if self.batch_mode == 'OFF':
            faces = itertools.chain.from_iterable(
                blender_utils.faces_from_mesh(
                    ob, global_matrix, self.use_mesh_modifiers)
                for ob in data_seq)

            rheo_stl_utils.write_stl(faces=faces, **keywords)
        elif self.batch_mode == 'OBJECT':
            if self.use_prefix:
                prefix = os.path.splitext(self.filepath)[0]
            else:
                prefix = os.path.dirname(self.filepath) + '/'

            keywords_temp = keywords.copy()
            for ob in data_seq:
                faces = blender_utils.faces_from_mesh(
                    ob, global_matrix, self.use_mesh_modifiers)
                keywords_temp["filepath"] = prefix + \
                    bpy.path.clean_name(ob.name) + ".stl"

                rheo_stl_utils.write_stl(
                    faces=faces, objectname=ob.name, **keywords_temp)

        return {'FINISHED'}


def menu_import(self, context):
    self.layout.operator(Import_RHEO_STL.bl_idname, text="Rheo Stl (.stl)")


def menu_export(self, context):
    self.layout.operator(Export_RHEO_STL.bl_idname, text="Rheo Stl (.stl)")


def register():

    bpy.utils.register_class(Export_RHEO_STL)

    # bpy.types.TOPBAR_MT_file_import.append(menu_import)
    bpy.types.TOPBAR_MT_file_export.append(menu_export)


def unregister():

    bpy.utils.unregister_class(Export_RHEO_STL)

    # bpy.types.TOPBAR_MT_file_import.remove(menu_import)
    bpy.types.TOPBAR_MT_file_export.remove(menu_export)


if __name__ == "__main__":
    register()
